﻿#requires -Version 2.0 -Modules ActiveDirectory

function Test-ReplicationMode
{
  <#
      .SYNOPSIS
      Test-ReplicationMode returns the Replication-Mode for the SYSVOL-Folder

      .DESCRIPTION
      The Script checks the Replication Mode of the SYSVOL-Folder. AD-Cmdlets must be available.

      .EXAMPLE
      Test-ReplicationMode
      Returns the ReplicationMode of the Domain

      .NOTES
      If the domain still uses FRS for SYSVOL-Replication, you can find the technet documentation migration here:
      https://blogs.technet.microsoft.com/filecab/2014/06/25/streamlined-migration-of-frs-to-dfsr-sysvol/

      .LINK
      URLs to related sites
      The first link is opened by Get-Help -Online Test-ReplicationMode
  #>


  [CmdletBinding()]
  param() 
  $DFSRGlobalSettings = Get-ADObject -Filter {
    name -eq 'DFSR-GlobalSettings' 
  } -Properties 'msDFSR-Flags'
  If ( $DFSRGlobalSettings.'msDFSR-Flags' -eq 48 ) 
  {
    'ReplicationMode DFS' 
  }
  Else 
  {
    'ReplicationMode FRS' 
  }
}

function Set-GPDebugLogging
{
  <#
      .SYNOPSIS
      Enables enhanced GPO Logging for the Client.

      .DESCRIPTION
      To Default Logging for Group Policies writes to the Windows Event Log. 
      For a deeper insight into the working of the Group Policy Client, enhanced GPO Logging can be activated. 
      The Log is written to %windir%\debug. 

      .EXAMPLE
      Set-GPDebugLogging 
      Enables Debug Logging (Sets the according Registry-Key)

      .EXAMPLE
      Set-GPDebugLogging -disable
      Disables Debug Logging (Sets the according Registry-Key)
  #>


  [CmdletBinding()]
  param(
    # To disable Debug-Logging, enabled this Parameter
    [switch]
    $disable
  )
  $diagnosticsPath = 'HKEY_LOCAL_MACHINE\Software\Microsoft\Windows NT\CurrentVersion\Diagnostics'
  If (-not ( Test-Path -Path Registry::$diagnosticsPath)) 
  { 
    New-Item -Path Registry::$diagnosticsPath 
    New-ItemProperty -Path Registry::$diagnosticsPath -Name GPSvcDebugLevel -Value '0x30002' -PropertyType DWord
  }
  New-Item -Path $env:windir\debug\Usermode\ -ItemType Directory
}

function Get-GPDebugLog 
{
  <#
      .SYNOPSIS
      Reads the GPSVC.Log File 

      .DESCRIPTION
      Get-GPDebuglog can parse the GPSVC.Log File and export its contents in readable form. 

      .EXAMPLE
      Get-GPDebugLog | out-gridview
      Reads the GPSVC.Log and returns it in a gridview

      .LINK
      URLs to related sites
      The first link is opened by Get-Help -Online Get-GPDebugLog

      .Notes
      A Great Tool for Troubleshooting GPSVC.Log: Policy Reporter
      http://www.sysprosoft.com/policyreporter.shtml
  #>


  [CmdletBinding()]
  param(
    # If gpsvc.log is not in the default-path, put it here
    [string]
    [ValidateScript({
          Test-Path -Path $_ -PathType Leaf 
    })]
    $gpSvcLogPath = "$env:windir\debug\UserMode\gpsvc.log", 
    
    # Format the DateTime Entries from the log - not implemented yet
    [string]$TimeFormat = 'HH:mm:ss:fff'
  )
 
  [RegEx]$LogID = '(GPSVC\(\S{3,4}.\S{3,4}\)) ((?:\d\d:){3}\d{3}) ([\s\S]{1,})'
  [RegEx]$ProcessID = '(?<=\()([A-Za-z\d]{2,4})(?=\.)' # '([A-Za-z\d]{3})(?=\.)'
  [RegEx]$ThreadID = '(?<=\.)([A-Za-z\d]{2,4})(?=\))' # '(?<=\.)([A-Za-z\d]{3})'
  $gpSvcLog = Get-Content -Path $gpSvcLogPath -Encoding Unicode -ReadCount 0
  $Index = 1

  ForEach ( $LogItem in $gpSvcLog )
  {
    $Null = $LogItem -match $LogID
    $LogEntry = @{
      Index     = $Index
      EntryType = $matches[1]
      Entrytime = $matches[2]
      Message   = $matches[3]
      ProcessID = [Convert]::ToInt32(($ProcessID.Matches($matches[1]).value),16)
      ThreadID  = [Convert]::ToInt32(($ThreadID.Matches($matches[1]).value),16)
    }
    New-Object -TypeName PSCustomObject -Property $LogEntry 
    $Index++
  }
}

function Get-GPExtensions {
  <#
      .SYNOPSIS
      Gets all installed Group Policy Extensions from the Registry

      .DESCRIPTION
      All Group Policy Extensions must be registered in the Registry.Get-GPExtension shows all Extensions.
      It can also show the Configuration-Settings of the Extensions by using the Parameter -Showregkeys. 

      .EXAMPLE
      Get-GPExtensions -ShowRegKeys
      Gets all Extension and their Settings
  #>


  [CmdletBinding()]
  param(
    [switch]
    $ShowRegKeys
  )
    $gpExtensionsPath = 'HKLM:\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Winlogon\GPExtensions'
    $SpalteDefault = @{ Name='Extension Name'
                        expression= { $_.'(Default)' } }
    $SpalteGUID = @{ Name='GUID'
                     expression = { $_.PSChildName }}
    $SelectParam = @{ Property=$SpalteGUID,$SpalteDefault,'ProcessGroupPolicy','DllName' }
    $Extensions = 'Guid',
             'Extension Name',
             'DllName',
             'ProcessGroupPolicy',
             'ProcessGroupPolicyEx',
             'GenerateGroupPolicy',
             'EnableAsynchronousProcessing',
             'ExtensionEventSource',
             'MaxNoGPOListChangesInterval',
             'NoBackgroundPolicy',
             'NoGPOListChanges',
             'NoMachinePolicy',
             'NoSlowLink',
             'NotifyLinkTransition',
             'NoUserPolicy',
             'PerUserLocalSettings',
             'RequiresSuccessfulRegistry'

    If ( $ShowRegKeys ) { 1 | Select-Object -Property $Extensions }
    Get-ChildItem -Path $gpExtensionsPath | ForEach-Object { 
       $Regkeys = $_ | Get-ItemProperty  
       If ( $ShowRegKeys )
       {
            $RegKeys | Select-Object -Property $SpalteGuid,$SpalteDefault,*
       }
       Else 
       {
         if ( $regkeys )
                { $RegKeys | Select-Object @SelectParam }
         Else {$_ | Select-Object @SelectParam }
       }
    }
}

function Get-EmptyGPO {
  <#
      .Synopsis
      Gets Empty GPO, Only Empty Computer-Settings or Only Empty User-Settings
      .DESCRIPTION
      This Cmdlet can find empty Group-Policy settings, either by name or through Pipeline.
      It has 3 Modes: Find all empty Policies, or find Policies where only Computer-Settings or User-Settings
      are enabled. 
      .EXAMPLE
      Get-EmptyGPO -name "MyEmptyGpo"
      Tests if MyEmptyGpo has Settings. If it has not, no output is generated, elsewise the Cmdlet returns the 
      Policy-Object 
      .EXAMPLE
      Get-EmptyGPO -name "MyEmptyGpo" -NoUsersettings
      Tests if MyEmptyGpo Has No User-Settings enabled, but Computer-Settings! If is completely empty, nothing
      is returned, elsewise the Policy-Object
      .EXAMPLE
      Get-GPO -ALL | Get-EmptyGPO
      Returns all empty Group-Policy-Objects from the Domain
      .EXAMPLE
      $EmptyGPOs = @(Get-GPO -ALL | Get-EmptyGPO)
      $EmptyComputerGPOs = @(Get-GPO -ALL | Get-EmptyGPO -NoComputerSettings)
      $EmptyGPOs = $EmptyGPOs + $EmptyComputerGPOs
      Returns all GPOs without Computer-Settings and alle completely empty GPOs

  #>
  
    # [CmdletBinding(DefaultParameterSetName='All')]
    param
    (
        # Name of the Group-Policy to examine
        [ValidateNotNullOrEmpty()]
        [Parameter(Mandatory = $true,
                   Position = 0,
                   ValueFromPipelineByPropertyName = $true,
                   HelpMessage = 'Enter the name of the Policy')]
        [Alias('displayname')]
        [string]$name,
   
        [Parameter(Mandatory = $true,
                   Position = 1,
                   Parametersetname = 'All')]
        [Validateset('All','NoComputerSettings','NoUsersettings')]
        [String]$Scope = 'All'
    )
    
    Begin
    {
        Import-Module -Name GroupPolicy    
    }

    Process
    {
        # Get an XML-Report of all Group-Policys. This may take a while, depending on the number of Policies
        [xml]$gpoReport = Get-GPOReport -ReportType Xml -name $name
        
        Switch ($Scope)
        {
            # Test for non-exisiting user.Extensiondata-Node
            'NoUserSettings' 
            {
                If ((-not $gpoReport.gpo.user.extensiondata) -And ($gpoReport.gpo.computer.extensiondata))
                   { Get-GPO -Name $name }                
            }
            # Test for non-exisiting Computer-Extensiondata-Node
            'NoComputerSettings' 
            {
                If (($gpoReport.gpo.user.extensiondata) -And (-not $gpoReport.gpo.computer.extensiondata))
                   { Get-GPO -Name $name }   
            }
            'All' 
            { 
                If ((-not $gpoReport.gpo.user.extensiondata) -And (-not $gpoReport.gpo.computer.extensiondata))
                   { Get-GPO -Name $name }
            }
        }
    }
}

function Get-EmptyGPOReport 
{
  <#
      .SYNOPSIS
      Analyses Group Policys for empty Computer- and/or User-Configuration

      .DESCRIPTION
      This Ceckup Policy checks a GPO for empty Computer- or User-Settings. You can Pipe GPOs for Analysis

      .EXAMPLE
      Get-EmptyGPOReport -name "Default Domain Policy" 
      Returns two columns, one for UserPolicy and one for ComputerPolicy

      Displayname           UserPolicy ComputerPolicy
      -----------           ---------- --------------
      Default Domain Policy Set        Set           

      .EXAMPLE
      Get-GPO -All | Get-EmptyGPOReport
  #>


  param 
  (
    # the name of the gpo to test
    [ValidateNotNullOrEmpty()]
    [Parameter(Mandatory = $true,HelpMessage='Display Name of the GPO',
    ValueFromPipelineByPropertyName = $true)]
    [Alias('displayname')]
    [string]$name
  )

  Begin
  {
    Import-Module -Name GroupPolicy
  }

  Process
  {
    $Computer = 'Empty'
    $User = 'Empty'
    [xml]$gpoReport = Get-GPOReport -ReportType Xml -Name $name

    If ( $gpoReport.GPO.Computer.ExtensionData ) 
    {
      $Computer = 'Set' 
    }

    If ( $gpoReport.GPO.User.ExtensionData ) 
    {
      $User = 'Set' 
    }    

    $GPOSettings = New-Object -TypeName PSObject -Property @{
      Displayname    = $gpoReport.gpo.Name
      ComputerPolicy = $Computer
      UserPolicy     = $User
    }    
    $GPOSettings | Select-Object -Property DisplayName,ComputerPolicy,UserPolicy
  }
}

function Get-UnlinkedGPOs
{
  <#
      .SYNOPSIS
      Find all unlinked Group-Policys in a domain

      .DESCRIPTION
      This function finds all unlinked Group Policys in a domain. The optional Parameter -domain defines where to search for unlinked GPOs. 

      .EXAMPLE
      Get-UnlinkedGPOs
      Starts a search for unlinked gpos in the current domain. 
  #>

  [CmdletBinding()]
  param(
    [string]
    $domain
  )
  
  If ( -not ( $domain ))
  { 
    $domainObject = Get-ADDomain
  }
  else
    { $domainObject = Get-ADDomain -Identity $domain }  
  Write-Verbose -Message ("{0}" -f $domainObject.LinkedGroupPolicyObjects)
  [regex]$GUID = '[A-Fa-f\d]{8}-[A-Fa-f\d]{4}-[A-Fa-f\d]{4}-[A-Fa-f\d]{4}-[A-Fa-f\d]{12}'
  [array]$Policies = $guid.matches( $domainObject.LinkedGroupPolicyObjects ).Value
  $Policies += $guid.matches(( Get-ADOrganizationalUnit -filter * ).LinkedGroupPolicyObjects).Value
  Get-GPO -all | Where-Object { $_.id -notin $Policies }
}



function Enable-AGPMEncryptedMail
{
  [CmdletBinding()]
  param(
    # Set this Parameter to Disable Mail Encryption
    [switch]
    $disable,
    
    [ValidateRange(0,65535)]
    [int]
    $port = 587 
  )

  $AGPMKey = 'HKLM:\Software\Microsoft\AGPM'
  If ( $disable ) 
  {
    $Encrypted = 0 
  }
  Else 
  {
    $Encrypted = 1 
  }
  Set-ItemProperty -Path $AGPMKey -Name EncryptSMTP -Value $Encrypted
  Set-Item -Path $AGPMKey -Path -Name -Path SmtpPort -Value $port
}

function Add-GPODefaultPermissions
{
  <#
      .SYNOPSIS
      Adds Additional Permissions to the GPO-Schema Template

      .DESCRIPTION
      If a new GPO is created, default Permissions are read from the AD-Schema. This function extends the schema with Read-Permissions for
      Authenticated Computers or custom SDDL-Strings can be added using the parameter -SDDLStringToAdd. The SDDL-Strings are explained in the 
      notes for this function.

      .EXAMPLE
      Add-GPODefaultPermissions
      Adds the Group "Domain Computers" with Read-Permissions to the Group Policy Container Default Permissions

      .EXAMPLE
      Add-GPODefaultPermissions -SDDLString "(A;CI;LCRPLORC;;;DU)"
      Adds the Group "Domain Computers" with Read-Permissions to the Group Policy Container Default Permissions

      .NOTES
      Meaning of SDDL-Flags:
      Access type: A = Access Allowed
      ACE flag: CI = Container Inherit
      Permissions:
      LC = List Contents
      RP = Read All Properties
      LO = List Object
      RC = Read Permissions
      Access subject: DC = Domain Computers
      More Infos: 
      https://msdn.microsoft.com/de-de/library/windows/desktop/aa379602(v=vs.85).aspx
      http://woshub.com/how-to-change-default-permissions-for-new-gpos/
  #>


  [CmdletBinding()]
  param (
    # The Permission to add to the GPO Container Object. Default is Authenticated Computers
    [String]
    $SDDLStringToAdd = '(A;CI;LCRPLORC;;;DC)'
  )
  
  $ADschemaDn = ( Get-ADRootDSE ).schemaNamingContext 
  $GPCClass = Get-ADObject -Filter { name -eq 'Group-Policy-Container' } -SearchBase $ADschemaDn -Properties DefaultSecurityDescriptor
  $NewGPCClassDescriptor = $GPCClass.DefaultSecurityDescriptor + $SDDLStringToAdd
  if ( -not $GPCClass.DefaultSecurityDescriptor.Contains( $SDDLStringToAdd ))
  { 
    Set-ADObject -Identity $GPCClass -Partition $ADschemaDn -Replace @{ DefaultSecurityDescriptor = $NewGPCClassDescriptor }
  }
  Update-SchemaCache
}

function Update-SchemaCache
{
  Try 
  {
    $SchemaMaster = (Get-ADForest).SchemaMaster.Split('.')[0]
    $RootDSE = [ADSI] "LDAP://$SchemaMaster/RootDSE"
    $RootDSE.put("SchemaUpdateNOW",1)
    $RootDSE.SetInfo()
    Write-Information -MessageData 'Update successfull'
  }
  Catch
  {
    $_.Exception.Message
  }
}
